;	===============================================================
;	ESTRUC.CPY - Entity Record Structures
;	===============================================================
;	Copyright 1995-1998 Evolution Computing
;	All rights reserved
;
;	This describes DBVERSION = 24
;	===============================================================

ET_Max		equ	41		;highest entity type #
ET_MinI		equ	80h		;range of valid non-drawing
ET_MaxI		equ	81h		;entities

;	===============================================================
;	Structure common to all entity records
;	(see IBMGR.CPY for EType=0 InfoBlocks - they don't have CStuff)
;	---------------------------------------------------------------
;	FCW version 5.05 added LWidth and Tag fields to all geometric
;	entities.
;	===============================================================

CStuff		struc
ERLen		dword	sizeof(CStuff)	;entity record length
EType		byte	0		;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..2.54 mm
;					;255 = thickness of SYMREF
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
CStuff		ends

EF_ERASED	equ	80h		;(old 5.163 files only!)
EF_Mark		equ	20h		;entity is highlighted
EF_NOSL		equ	10h		;ignore sublist (const SYMREF)

EF_CSREF	equ	08h		;color = color of SYMREF
EF_Share	equ	04h		;shared use flag

EF_A1		equ	02h		;arrow at T=1 end
EF_A0		equ	01h		;arrow at T=0 end

EF2_NOUTL	equ	01h		;no outline on fill

;	===============================================================
;	Entity class flags (returned by ECLASS handler in EAX)
;	===============================================================

;	Bits 0 and 1 specify how many parameters required to describe
;	a point within the entity

C_NONE	equ	00000h		;no point data in entity
C_PNT	equ	00000h		;point data
C_LIN	equ	00001h		;entity is linear (2D/3D line, arc, spline)
C_SRF	equ	00002h		;entity has surface (open cyl, filled poly)
C_VOL	equ	00003h		;entity encloses volume (sphere, solid)

C_FILL	equ	00004h		;bit 2 set=interior is solid or filled
C_3D	equ	00008h		;bit 3 set=3D, clear=2D
C_2D	equ	00000h		;bit 3 set=3D, clear=2D

C_SAFE	equ	08000h		;extended SVC functions safe to call
;				;(not necessarily implemented, but
;				; safe to call - has limit check code).
;				;These are functions EO_RENDER and above.

C_CLS	equ	00080h		;bit 7 set=closed (circle or poly)
C_ANN	equ	00100h		;bit 8 set=annotation only, clear=model
C_COMP	equ	00200h		;bit 9 set=composite entity
C_EDGE	equ	00400h		;bit 10 set = entity supports EEdge svc
C_PLACE	equ	00800h		;supports EPlace

;	===============================================================
;	Entity handler support flags (returned by ECLASS handler in EDX)
;	===============================================================
;
H_TRIM	equ	00001h		;supports trim
H_BND	equ	00002h		;supports bend (not used)
H_CUT	equ	00004h		;supports cut
H_DYNED	equ	00008h		;supports graphic edit (not used)
H_MATH	equ	00010h		;supports near, tan, t, etc.
H_MATH3	equ	00020h		;supports 3d near, tan, t, etc.
H_NONE	equ	00000h		;supports nothing

;	===============================================================
;	Point2: 2d point entity
;	---------------------------------------------------------------

ET_Point2	equ	1		;entity type code
C_Point2	equ	C_PNT
H_Point2	equ	H_NONE

Point2		struc
ERLen		dword	sizeof(Point2)	;record length
EType		byte	ET_Point2	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
X		real4	0.0
Y		real4	0.0
Point2		ends

;	---------------------------------------------------------------
;	Line2: 2D line entity
;	---------------------------------------------------------------

ET_Line2	equ	2		;entity type code
C_Line2		equ	C_LIN+C_EDGE+C_SAFE
H_Line2		equ	H_MATH+H_TRIM+H_CUT

Line2		struc
ERLen		dword	sizeof(Line2)	;record length
EType		byte	ET_Line2	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
X1		real4	0.0		;start point
Y1		real4	0.0
X2		real4	0.0		;end point
Y2		real4	0.0
Line2		ends

;	---------------------------------------------------------------
;	EType 3: 2D Path/Spline (open or closed) entity data
;		 Length = overhead+Len(node list)
;	---------------------------------------------------------------

ET_Path2	equ	3		;entity type code
EL_Path2	equ	Path2.Count	;length without node list
C_Path2		equ	C_LIN+C_EDGE+C_SAFE
H_Path2		equ	H_MATH+H_CUT+H_TRIM
;
ET_IDPath2	equ	31		;future separate entity id #s
ET_IDPoly2	equ	32
ET_IDSpline2	equ	33
ET_IDSPoly2	equ	34
ET_IDBezier2	equ	35
;
Path2		struc
ERLen		dword	EL_Path2	;entity record overhead
EType		byte	ET_Path2	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;---------------------------------------------- rest matches GPath2
SmType		db	SM_NO		;smoother type
SRes		db	8		;smoothing resolution
SParm		REAL4	0.0		;start parameter
EParm		REAL4	1.0		;end parameter
;					;full draw = # nodes-1 if open
;					;          = # nodes if closed
;---------------------------------------------- rest matches GNL2
Count		word	0		;# nodes in list
Flags		db	0		;nodelist control flags
		db	0		;unused
X		REAL4	0.0		;first node
Y		REAL4	0.0
Path2		ends

;	Values for Flags
;
;	NL_CLS	equ	80h	;1 = close node list (polygon)
;	NL_SLD	equ	40h	;1 = interior is solid (filled)
;	NL_2D	equ	00h	;2D node list (8 bytes per node)
;	NL_2DW	equ	01h	;2D node list with W parm (12 bytes per node)

;	Note that SParm..EParm ranges 0..Count-1 for a path
;	Note that SParm..EParm ranges 0..Count for a poly

;	Values for Smoother Flags
;
;	SM_NO	equ	0	;no smoothing
;	SM_CB	equ	1	;Cubic B-Spline (as in FastCAD 2.x)
;	SM_PB	equ	2	;Parabolic Blend (through-point)
;	SM_BZ	equ	3	;Bezier
;
;	---------------------------------------------------------------
;	Box2: specialized version of 2D Path/Spline entity data
;	---------------------------------------------------------------

Box2		struc
ERLen		dword	sizeof(Box2)	;entity record overhead
EType		byte	ET_Path2	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;---------------------------------------------- rest matches GPath2
SmType		db	SM_NO		;smoother type
SRes		db	8		;smoothing resolution
SParm		REAL4	0.0		;start parameter
EParm		REAL4	4.0		;end parameter
;---------------------------------------------- rest matches GNL2
Count		word	4		;# nodes in list
Flags		db	NL_CLS		;nodelist control flags
		db	0		;unused
X1		REAL4	0.0
Y1		REAL4	0.0
X2		REAL4	1.0
Y2		REAL4	0.0
X3		REAL4	1.0
Y3		REAL4	1.0
X4		REAL4	0.0
Y4		REAL4	1.0
Box2		ends

;	---------------------------------------------------------------
;	Etype 4: XP Custom Entity
;	---------------------------------------------------------------

ET_XP		equ	4		;entity type code
C_XP		equ	0
H_XP		equ	0

XPEnt		struc
ERLen		dword	sizeof(XPEnt)	;record length
EType		byte	ET_XP		;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
XPID		word	XPID		;XP ID # for custom entity SVC
;		-------------------	;custom entity data follows
XType		byte	0		;entity sub-type (if needed)
XFlags		byte	0		;sub-type flags (if needed)
;					;xp-specific data starts here
XPEnt		ends

;	---------------------------------------------------------------
;	Etype 5: 2d Text
;	---------------------------------------------------------------

ET_Text2	equ     5               ;entity type code
EL_Text2	equ	sizeof(Txt2)    ;entity length (overhead)
C_Text2		equ	C_PNT+C_ANN+C_EDGE
H_Text2		equ	H_MATH
;
Txt2		struc
ERLen 		DWORD   EL_Text2        ;entity record length
EType  		byte    ET_Text2        ;entity type code
EFlags 		byte    0               ;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
Font    	word	0               ;text font id #
X		REAL4	0.0		;text origin
Y		REAL4	0.0
Hght		REAL4	0.2		;text height
XScl		REAL4	1.0		;text cell X scale (horiz stretch)
BAng		REAL4	0.0		;text baseline angle
Spacing 	REAL4	0.0		;inter-line spacing
TFlags  	WORD    0               ;style flags
DFlags  	WORD    0               ;data handling flags
Just    	WORD    0               ;text justification code
TData		SBYTE	'S'		;text data ASCIIZ string
;					;needed for font samples
Txt2		ends

;	---------------------------------------------------------------
;	Cir2: 2D Circle entity
;	---------------------------------------------------------------

ET_Cir2		equ	6		;entity type code
C_Cir2		equ	C_LIN+C_CLS+C_EDGE+C_SAFE
H_Cir2		equ	H_MATH+H_CUT

Cir2		struc
ERLen		dword	sizeof(Cir2)
EType		byte	ET_Cir2		;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
CenX		real4	0.0		;circle center
CenY		real4	0.0
Radius		real4	1.0		;circle radius
Cir2		ends

;	---------------------------------------------------------------
;	EType 7: 2D Arc
;	---------------------------------------------------------------

ET_Arc2		equ	7
EL_Arc2		equ	sizeof(Arc2)
C_Arc2		equ	C_LIN+C_EDGE+C_SAFE
H_Arc2		equ	H_MATH+H_CUT+H_TRIM

Arc2		struc
ERLen		dword	sizeof(Arc2)
EType		byte	ET_Arc2		;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
CenX		real4	0.0		;arc center
CenY		real4	0.0
Radius		real4	1.0		;arc radius
SAng		real4	0.0		;start angle, radians
AngW		real4	0.0		;arc width
Arc2		ends

;	---------------------------------------------------------------
;	EType 8: 2d Ellipse
;	---------------------------------------------------------------

ET_Elp2		equ	8
C_Elp2		equ	C_LIN+C_CLS+C_EDGE+C_SAFE
H_Elp2		equ	H_MATH+H_CUT

Elp2		struc
ERLen		dword	sizeof(Elp2)	;entity record length
EType		byte	ET_Elp2		;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
CenX		real4	0.0		;center
CenY		real4	0.0
Radius		real4	1.0		;major radius
Ecc		real4	0.0		;eccentricity minor/major (0..1)
Incl		real4	0.0		;major axis inclination angle 0..pi
Elp2		ends
;
;	---------------------------------------------------------------
;	EType 9: 2d Elliptical Arc
;	---------------------------------------------------------------

ET_ElA2		equ	9
EL_ElA2		equ	sizeof(ElA2)
C_ElA2		equ	C_LIN+C_EDGE+C_SAFE
H_ElA2		equ	H_MATH+H_CUT+H_TRIM

ElA2		struc
ERLen		dword	sizeof(ElA2)	;entity record length
EType		byte	ET_ElA2		;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
CenX		real4	0.0		;center
CenY		real4	0.0
Radius		real4	1.0		;major radius
Ecc		real4	0.0		;eccentricity minor/major (0..1)
Incl		real4	0.0		;major axis inclination angle 0..pi
SAng		real4	0.0		;start angle 0..2pi
AngW		real4	0.0		;arc width 0..2pi
ElA2		ends

;	---------------------------------------------------------------
;	Etype 10: 2d Action Text (HotSpot & its macro text)
;	---------------------------------------------------------------

ET_Act2		equ     10               ;entity type code
EL_Act2		equ	sizeof(Act2)    ;entity length (overhead)
C_Act2		equ	C_PNT+C_ANN+C_EDGE
H_Act2		equ	H_MATH
;
Act2		struc
ERLen 		DWORD   EL_Act2        ;entity record length
EType  		byte    ET_Act2        ;entity type code
EFlags 		byte    0               ;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
HotX1		real4	0.0		;lower-left corner of hotspot
HotY1		real4	0.0
HotX2		real4	1.0
HotY2		real4	1.0
;
Font    	word	0               ;text font id #
X		REAL4	0.0		;text origin
Y		REAL4	0.0
Hght		REAL4	0.2		;text height
XScl		REAL4	1.0		;text cell X scale (horiz stretch)
BAng		REAL4	0.0		;text baseline angle
Spacing 	REAL4	0.0		;inter-line spacing
TFlags  	WORD    0               ;style flags
DFlags  	WORD    0               ;data handling flags
Just    	WORD    0               ;text justification code
TData		SBYTE	0		;text data ASCIIZ string
Act2		ends

;	===============================================================
;	CtrlP: Control Points Entity
;	===============================================================

ET_CtrlP	equ	12		;entity type code
C_CtrlP		equ	0
H_CtrlP		equ	0

CtrlP		struc
ERLen		dword	sizeof(CtrlP)	;record length
EType		byte	ET_CtrlP	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
X1		real4	0.0		;ctrl point 1
Y1		real4	0.0
X2		real4	0.0		;ctrl point 2
Y2		real4	0.0
X3		real4	0.0		;ctrl point 3
Y3		real4	0.0
CFlags		dword	CP_ALN+CP_CUT	;control type flags
CText		db	0		;control text
CtrlP		ends

;	Values for CtrlP.CFlags:

CP_ALN		equ	1		;align bearing with p1->p2
CP_CUT		equ	2		;cut at p1->p2
CP_SCALEY	equ	4		;scale Y to fit
CP_UDSCALE	equ	8		;use dyntrak scale
CP_OFFSET	equ	16		;offset from place point

;	===============================================================
;	Wall2: 2d wall entity
;	---------------------------------------------------------------

ET_Wall2	equ	13		;entity type code
C_Wall2		equ	C_EDGE
H_Wall2		equ	H_NONE

Wall2		struc			;2d wall header
ERLen		dword	sizeof(Wall2)	;record length
EType		byte	ET_Wall2	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
NextID		word	0		;next node id
;
;	list of nodes goes here:
;
FirstNode	word	0		;ID=0=end of node list
Wall2		ends

Wall2N		struc			;2d wall node record
ID		word	0		;node id #
nSpans		word	0		;# spans connected to this node
nVisit		word	0		;traversal counter
X		real4	0.0		;node location
Y		real4	0.0
Wall2N		ends

Wall2S		struc			;2d wall node span exit record
ToID		word	0		;exit to node #
flags		db	0
Thickness	real4	0.0		;exit thickness
Ang		real4	0.0		;exit angle
XData		dword	0		;extra data
Wall2S		ends

;	Values for Wall2S.flags

Wall_Exit	equ	1		;span exit has been taken

Wall_CF		equ	0		;span is center from
Wall_CT		equ	0		;span is center to
Wall_LF		equ	4		;span is left from
Wall_LT		equ	8		;span is left to
Wall_RF		equ	8		;span is right from
Wall_RT		equ	4		;span is right to

;	---------------------------------------------------------------
;	EType 14:  3D line entity
;	---------------------------------------------------------------

ET_Line3	equ	14		;entity type code
C_Line3		equ	C_LIN+C_3D+C_SAFE
H_Line3		equ	H_MATH3

Line3		struc
ERLen		dword	sizeof(Line3)	;record length
EType		byte	ET_Line3	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
X1		real4	0.0		;start point
Y1		real4	0.0
Z1		real4	0.0
X2		real4	0.0		;end point
Y2		real4	0.0
Z2		real4	0.0
Line3		ends

;	---------------------------------------------------------------
;	EType 15: 3D Path/Spline (open or closed) entity data
;		 Length = overhead+Len(node list)
;	---------------------------------------------------------------

ET_Path3	equ	15		;entity type code
EL_Path3	equ	Path3.Count	;length without node list
C_Path3		equ	C_LIN+C_EDGE+C_3D+C_SAFE
H_Path3		equ	H_MATH3
;
Path3		struc
ERLen		dword	EL_Path3	;entity record overhead
EType		byte	ET_Path3	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;---------------------------------------------- rest matches GPath2
SmType		db	SM_NO		;smoother type
SRes		db	8		;smoothing resolution
SParm		REAL4	0.0		;start parameter
EParm		REAL4	1.0		;end parameter
;					;full draw = # nodes-1 if open
;					;          = # nodes if closed
;---------------------------------------------- rest matches GNL3
Count		word	0		;# nodes in list
Flags		db	0		;nodelist control flags
		db	0		;unused
X		REAL4	0.0		;first node
Y		REAL4	0.0
Z		REAL4	0.0
Path3		ends

;	Values for Flags
;
;	NL_CLS	equ	80h	;1 = close node list (polygon)
;	NL_SLD	equ	40h	;1 = interior is solid (filled)
;	NL_3D	equ	02h	;3D node list (12 bytes per node)
;	NL_3DW	equ	03h	;3D node list with W parm (16 bytes per node)

;	Note that SParm..EParm ranges 0..Count-1 for a path
;	Note that SParm..EParm ranges 0..Count for a poly

;	Values for Smoother Flags
;
;	SM_NO	equ	0	;no smoothing
;	SM_CB	equ	1	;Cubic B-Spline (as in FastCAD 2.x)
;	SM_PB	equ	2	;Parabolic Blend (through-point)
;	SM_BZ	equ	3	;Bezier
;

Rect3		struc
ERLen		dword	sizeof(Rect3)	;entity record overhead
EType		byte	ET_Path3	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;---------------------------------------------- rest matches GPath2
SmType		db	SM_NO		;smoother type
SRes		db	8		;smoothing resolution
SParm		REAL4	0.0		;start parameter
EParm		REAL4	4.0		;end parameter
;					;full draw = # nodes-1 if open
;					;          = # nodes if closed
;---------------------------------------------- rest matches GNL3
Count		word	4		;# nodes in list
Flags		db	NL_CLS+NL_SLD+NL_3D ;nodelist control flags
		db	0		;unused
P1		GPnt3	<>
P2		GPnt3	<>
P3		GPnt3	<>
P4		GPnt3	<>
Rect3		ends

;	---------------------------------------------------------------
;	EType 17: 2D Multi-poly entity data (header)
;	---------------------------------------------------------------

ET_MPoly2	equ	17		;entity type code
L_MPoly2	equ	sizeof(MPoly2)	;entity initial overhead
C_MPoly2	equ	C_SRF+C_COMP+C_CLS+C_2D+C_FILL+C_EDGE
H_MPoly2	equ	0		;no math

MPoly2		struc
ERLen		dword	sizeof(MPoly2)	;record length
EType		byte	ET_MPoly2	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
MFlags		word	0		;multipoly flags
MPoly2		ends

;	A multipoly always has a sublist of the entities defining
;	its edges.

;	---------------------------------------------------------------
;	EType 18: 2D Part or Group (header)
;	---------------------------------------------------------------

ET_Part		equ	18		;entity type code
L_Part		equ	sizeof(Group)	;entity initial overhead
C_Part		equ	0
H_Part		equ	0		;no math

Part		struc
ERLen		dword	sizeof(Part)	;record length
EType		byte	ET_Part		;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
Part		ends

;	A part/group always has a sublist containing the entities
;	contained in the part/group. This can include nested groups.

;	When a part/group is unlocked, its entities are given a group
;	ID # and exploded. This ID # is used to rebuild a group
;	when it is again locked. This is CStuff.GroupID in
;	all entities.

;	---------------------------------------------------------------
;	Etype 20: DimL2N - New linear dimension entity 
;	---------------------------------------------------------------

ET_DimL2N	equ	20		;entity type code
C_DimL2N	equ	C_PNT+C_ANN
H_DimL2N	equ	0

DimL2N		struc
ERLen		dword	sizeof(DimL2N)	;entity record length
EType		byte	ET_DimL2N	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
DTFont		dw	0		;dimension text font ID
AStyle		dw	0		;arrowhead style
DFlags		dw	0		;style flags
DTHgt		real4	0.250		;dimension text height
LdrExt		real4	0.18		;leader extends beyond dimension line
LdrOfs		real4	0.0625		;leader offset from geometry
ALen		real4	0.18		;arrowhead length
AHgt		real4	0.08		;arrowhead height
TolP		real4	0.00		;+tolerance
TolM		real4	0.00		;-tolerance
Spacing		real4	0.0		;baseline spacing
DTDecP		db	3		;number of decimal places (0 to 9)
DTFmt		db	0		;prefix stg,0,suffix stg,0
		dd	0,0,0,0,0,0,0,0	;(room for 31 characters max)
;
;
NCount		word	1		;number of dimension sets
;
X1		REAL4	0.0		;dimension point 1 (geo end)
Y1		REAL4	0.0
X3		REAL4	0.0		;dimension point 3 (arw end 1)
Y3		REAL4	0.0
;
;	The following DimL2Set repeats for NCount times:
;
X2		REAL4	0.0		;dimension point 2 (geo end)
Y2		REAL4	0.0
X4		REAL4	0.0		;dimension point 4 (arw end 2)
Y4		REAL4	0.0
TPosX		REAL4	0.0		;forced text position 
TPosY		REAL4	0.0
;					;both = 0FFFFFFFFh if not forced
TValue		REAL4	0.0		;forced dim display value if <>0.0
;					;else display dim=Dist2P(P2->P4)
DimL2N		ends

DimL2Set	struc
X2		REAL4	0.0		;dimension point 2 (geo end)
Y2		REAL4	0.0
X4		REAL4	0.0		;dimension point 4 (arw end 2)
Y4		REAL4	0.0
TPosX		REAL4	0.0		;forced text position 
TPosY		REAL4	0.0
TValue		REAL4	0.0		;forced dim display value if <>0.0
;					;else display dim=Dist2P(P2->P4)
DimL2Set	ends

;	The following DFlags values are valid for linear dimensions:
;	(Actual definition is in DStyles.CPY)

;	DS2_CEN		equ	00h	;text centered in dimension line
;	DS2_OVR		equ	01h	;text over dimension line/arc
;	DS2_UND		equ	02h	;text under dimension line/arc
;	DS2_LEFT	equ	04h	;text to left of dim line/arc
;	DS2_RIGHT	equ	08h	;text to right of dim line/arc
;	DS2_TXALN	equ	10h	;1=dim text aligned/0=always horiz
;	DS2_ARWIN	equ	20h	;force arrows in (no autoflip)
;	DS2_TOL		equ	40h	;specify tolerancing
;	DS2_ACAD	equ	80h	;source was ACAD
;	DS2_SCALE	equ	400h	;scale dim value by 
;					;DStyle2.Spacing value
;
;	Shared DFlags bit values valid for linear dimensions only:

;	DS2_PRL		equ	1000h	;parallel dimension

;	---------------------------------------------------------------
;	Etype 21: DimA2 - NEW angular dimension entity
;	---------------------------------------------------------------

ET_DimA2N	equ	21		;entity type code
C_DimA2N	equ	C_PNT+C_ANN
H_DimA2N	equ	0
	
DimA2N		struc
ERLen		dword	sizeof(DimA2N)	;entity record length
EType		byte	ET_DimA2N	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
DTFont		dw	0		;dimension text font ID
AStyle		dw	0		;arrowhead style
DFlags		dw	0		;style flags
DTHgt		real4	0.250		;dimension text height
LdrExt		real4	0.18		;leader extends beyond dimension line
LdrOfs		real4	0.0625		;leader offset from geometry
ALen		real4	0.18		;arrowhead length
AHgt		real4	0.08		;arrowhead height
TolP		real4	0.00		;+tolerance
TolM		real4	0.00		;-tolerance
Spacing		real4	0.0		;baseline spacing
DTDecP		db	3		;number of decimal places (0 to 9)
DTFmt		db	0		;prefix stg,0,suffix stg,0
		dd	0,0,0,0,0,0,0,0	;(room for 31 characters max)
;
L1Rad		REAL4	0.0		;dist to leader 1 start from center
;
NCount		word	1		;number of dimension sets
;
;	The following repeats for NCount sets
;
CenX		REAL4	0.0		;arc center
CenY		REAL4	0.0
Radius		REAL4	0.0		;arc radius (dist to arrow point)
SAng		REAL4	0.0		;start angle, radians
AngW		real4	0.0		;arc width
;
L2Rad		REAL4	0.0		;dist to leader 2 start from center
TPosX		real4	0.0		;user-spec text position
;					;=0FFFFFFFFh if none (DFlags bit D_UTP
TPosY		real4	0.0
TValue		real4	0.0		;user-fixed value (0.0=none)
;
DimA2N		ends

;	The following DFlags values are valid for angular dimensions:
;	(Actual definition is in DStyles.CPY)

;	DS2_CEN		equ	00h	;text centered in dimension line
;	DS2_OVR		equ	01h	;text over dimension line/arc
;	DS2_UND		equ	02h	;text under dimension line/arc
;	DS2_TXALN	equ	10h	;1=dim text aligned/0=always horiz
;	DS2_ARWIN	equ	20h	;force inside arrows
;	DS2_TOL		equ	40h	;specify tolerancing
;	DS2_ACAD	equ	80h	;source was ACAD
;	DS2_RDCM	equ	100h	;center mark
;	DS2_RDCL	equ	200h	;center lines

;	code actually uses
;	D_UTP		equ	100h	;fixed text position

;	DS2_L1		equ	1000h	;has starting leader
;	DS2_L2		equ	2000h	;has ending leader

;	---------------------------------------------------------------
;	Etype 22: DimC2N - New Circular dimension entity
;	---------------------------------------------------------------

ET_DimC2N	equ	22		;entity type code
C_DimC2N	equ	C_PNT+C_ANN
H_DimC2N	equ	0

DimC2N		struc
ERLen		dword	sizeof(DimC2N)	;entity record length
EType		byte	ET_DimC2N	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
DTFont		dw	0		;dimension text font ID
AStyle		dw	0		;arrowhead style
DFlags		dw	0		;style flags
DTHgt		real4	0.250		;dimension text height
LdrExt		real4	0.18		;leader extends beyond dimension line
LdrOfs		real4	0.0625		;leader offset from geometry
ALen		real4	0.18		;arrowhead length
AHgt		real4	0.08		;arrowhead height
TolP		real4	0.00		;+tolerance
TolM		real4	0.00		;-tolerance
Spacing		real4	0.0		;baseline spacing
DTDecP		db	3		;number of decimal places (0 to 9)
DTFmt		db	0		;prefix stg,0,suffix stg,0
		dd	0,0,0,0,0,0,0,0	;(room for 31 characters max)
;
TPosX		REAL4	0.0		;forced text position 
TPosY		REAL4	0.0
TValue		REAL4	0.0		;forced display value
;
CenX		REAL4	0.0		;center
CenY		REAL4	0.0
Radius		REAL4	0.0		;radius
LdrAng		REAL4	0.0		;leader angle, radians
LdrLen		REAL4	0.0		;leader length (outside of conic)
;
DimC2N	ends

;	The following DFlags values are valid for circular dimensions:
;	(Actual definition is in DStyles.CPY)

;	DS2_TXALN	equ	10h	;1=dim text aligned/0=always horiz
;	DS2_ARWIN	equ	20h	;force inside arrows
;	DS2_TOL		equ	40h	;specify tolerancing
;	DS2_RDCM	equ	100h	;center mark
;	DS2_RDCL	equ	200h	;center lines
;	DS2_SCALE	equ	400h	;scale dim value by 
;					;DStyle2.Spacing value

;	Shared DFlags bit values valid for circular dimensions only:

;	DS2_DIA		equ	4000h	;diameter (else radius)
;	DS2_LDR		equ	8000h	;dimension has a leader

;	---------------------------------------------------------------
;	Etype 23: DimO2N - New ordinate dimension entity 
;	---------------------------------------------------------------

ET_DimO2N	equ	23		;entity type code
C_DimO2N	equ	C_PNT+C_ANN
H_DimO2N	equ	0
	
DimO2N		struc
ERLen		dword	sizeof(DimO2N)	;entity record length
EType		byte	ET_DimO2N	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
DTFont		dw	0		;dimension text font ID
AStyle		dw	0		;arrowhead style
DFlags		dw	0		;style flags
DTHgt		real4	0.250		;dimension text height
LdrExt		real4	0.18		;leader extends beyond dimension line
LdrOfs		real4	0.0625		;leader offset from geometry
ALen		real4	0.18		;arrowhead length
AHgt		real4	0.08		;arrowhead height
TolP		real4	0.00		;+tolerance
TolM		real4	0.00		;-tolerance
Spacing		real4	0.0		;baseline spacing
DTDecP		db	3		;number of decimal places (0 to 9)
DTFmt		db	0		;prefix stg,0,suffix stg,0
		dd	0,0,0,0,0,0,0,0	;(room for 31 characters max)
;
nrldrs		word	0		;# leader lines
;
odbsx		real4	0.0		;baseline origin
odbsy		real4	0.0
odorgl		real4	0.0		;distance to geometry origin
;
;	For each leader, there is a set of two values:
;
odispx		real4	0.0		;distance from odbsx,odbsy
odispy		real4	0.0		;leader length
;
DimO2N		ends

;	The following DFlags values are valid for ordinate dimensions:
;	(Actual definition is in DStyles.CPY)

;	DS2_TOL		equ	40h	;specify tolerancing

;	Shared DFlags bit values valid for ordinate dimensions only:

;	DS2_VBase	equ	0000h	;vertical baseline
;	DS2_HBase	equ	1000h	;horizontal baseline
;	DS2_NDelta	equ	2000h	;Negative At-Org Delta
;	DS2_ODShift	equ	12	;shift to put hbase/vbase/ndelta
;					;at bits 1 & 2
;
;	q0 is a rising vertical odim
;	int q = (pEntRec->DimO2N.DFlags & DS2_RQuad)>>DS2_ODShift;
;	q = (q+1)&3;
;	float bang = q*HALFPI;

;	---------------------------------------------------------------
;	Etype 28: SYMDEF Symbol Definition
;	---------------------------------------------------------------

ET_SYMDEF	equ	28		;entity type code
C_SYMDEF	equ	0
H_SYMDEF	equ	0

SYMDEF		struc
ERLen		dword	sizeof(SYMDEF)	;record length
EType		byte	ET_SYMDEF	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
XLow		real4	0.0		;lowest x,y,z extents coordinate
YLow		real4	0.0
ZLow		real4	0.0
XHi		real4	0.0		;highest x,y,z extents coordinate
YHi		real4	0.0
ZHi		real4	0.0
;
flags		dword	0		;options/status flags
SName		db	32 dup (0)	;ANSIZ symbol name
SYMDEF		ends

SYMEND		struc			;SYMDEF end marker record
ERLen		dword	4
SYMEND		ends

;	A symbol definition usually has a sublist containing the
;	entities defining the symbol. If it does not, and SYMDEF.EFlags
;	has the EF_NOSL list set, then the SYMDEF is a memory constant
;	list (such as the arrowhead definitions table) where the
;	definition entities are stored immediately following the
;	SYMDEF record, and the entity list ends with a SYMEND record.

;	---------------------------------------------------------------
;	Etype 29: SYMREF Symbol Reference
;	---------------------------------------------------------------

ET_SYMREF	equ	29		;entity type code
C_SYMREF	equ	0
H_SYMREF	equ	0

SYMREF		struc
ERLen		dword	sizeof(SYMREF)	;record length
EType		byte	ET_SYMREF	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
XLow		real4	0.0		;lowest x,y,z extents coordinate
YLow		real4	0.0
ZLow		real4	0.0
XHi		real4	0.0		;highest x,y,z extents coordinate
YHi		real4	0.0
ZHi		real4	0.0
;
SName		db	32 dup (0)	;ANSIZ symbol name
DefAdr		dword	0		;run-time adrs of definition
TMat		real4 1.0,0.0,0.0,0.0, 0.0,1.0,0.0,0.0, 0.0,0.0,1.0,0.0
;					;SymDef -> World Coordinates TMAT
SYMREF		ends

SYMREFN		struc
ERLen		dword	sizeof(SYMREF)	;record length
EType		byte	ET_SYMREF	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
XLow		real4	0.0		;lowest x,y,z extents coordinate
YLow		real4	0.0
ZLow		real4	0.0
XHi		real4	0.0		;highest x,y,z extents coordinate
YHi		real4	0.0
ZHi		real4	0.0
;
SName		db	32 dup (0)	;ANSIZ symbol name
DefAdr		dword	0		;run-time adrs of definition
TM		SymTMat	<>
;					;SymDef -> World Coordinates TMAT
SYMREFN		ends


;	If a symbol reference has attributes, the attribute
;	reference entities are contained in a sublist of
;	the symbol reference entity.

;	---------------------------------------------------------------
;	Etype 30: 2d Text Attribute
;	---------------------------------------------------------------

ET_Atrib2	equ     30              ;entity type code
EL_Atrib2	equ	sizeof(Atrib2)  ;entity length (overhead)
C_Atrib2	equ	C_PNT+C_ANN
H_Atrib2	equ	H_MATH
;
Atrib2		struc
ERLen 	 	DWORD   EL_Atrib2       ;entity record length
EType   	byte    ET_Atrib2       ;entity type code
EFlags  	byte    0               ;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
AtFlags		word	0		;attribute flags
TagOfst		dword	0		;offset of Tag Text
PrmpOfst	dword	0		;offset of prompt text
ValOfst		dword	0		;offset of Value Text
;
Font   		word	0               ;text font id #
X		REAL4	0.0		;text origin
Y		REAL4	0.0
Hght		REAL4	0.2		;text height
XScl		REAL4	1.0		;text cell X scale (horz stretch)
BAng		REAL4	0.0		;text baseline angle
Spacing		REAL4	0.0		;inter-line spacing
TFlags 		WORD    0               ;style flags
DFlags 		WORD    0               ;data handling flags
Just    	WORD    0               ;text justification code
;
Atrib2		ends

;	Values for Atrib2.AtFlags:

AT_NOPRMP	equ	1		;constant value - no prompting
AT_HIDDEN	equ	2		;attribute not displayed
AT_DSPTAG	equ	4		;always display tag
AT_REF		equ	8000h		;atrib is in SYMREF
AT_NOTRAN	equ	8		;no SYMREF transform

;	The Atrib2 header structure is followed by 0 or more
;	ANSIZ text strings, whose offsets from Atrib2.ERLen
;	are stored in TagOfst, ValOfst, and PrmpOfst. The Tag
;	text is stored first if it exists. (The first stored
;	text is displayed when the attribute is drawn outside
;	a symbol reference.) All strings are followed by a 00-NULL
;	end of string list byte.

;	All strings and their terminating NULL bytes, as well as
;	the end of string list byte, are included in the final
;	entity length.

;	When a symbol containing an attribute is inserted,
;	the prompt text is displayed and the data value is
;	entered. The Value text from the original attribute
;	definition is used as the default if no text is entered.
;	The symbol when drawn displays the value text.
;	If no prompt text is specified, the tag text is used as
;	a prompt.

;	If Atrib2.AtFlags has the AT_DSPTAG flag set, the attribute
;	always displays the tag next instead of the value text.

;	If Atrib2.AtFlags has the AT_NOPRMP flag set, the value
;	specified in the attribute definition is used and no
;	prompting occurs.

;	If Atrib2.AtFlags has the AT_HIDDEN flag set, no attribute
;	text is displayed in the SYMBOL REFERENCE.

;	Attributes in SYMDEFs are copied/included_in each SYMREF,
;	with the VALUE text first, (from default/entry etc.).
;	These copies are drawn in place of the Attributes in the SYMDEF.

;	===============================================================
;	EType 31..36: Reserved for change to path poly id #s
;	---------------------------------------------------------------

;	===============================================================
;	Etype 40: XREF Definition
;	===============================================================

ET_XREF		equ	40		;entity type code
C_XREF		equ	0
H_XREF		equ	0

XREF		struc
ERLen		dword	sizeof(XREF)	;record length
EType		byte	ET_XREF		;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
XFlags		dword	0		;XREF flags (not used yet)
hDList		dword	0		;handle to the Drawing List
;
TMat		real4 1.0,0.0,0.0,0.0, 0.0,1.0,0.0,0.0, 0.0,0.0,1.0,0.0
;					;SymDef -> World Coordinates TMAT
;
FileName	db	260 dup (0)	;source file name
XREF		ends

;	===============================================================
;	Non-Drawing Entities - EType 80h ..0FFh
;	===============================================================
;	PASM: Print Assembly Main Entity Structure
;	---------------------------------------------------------------
PAVer		equ	0		;Structure Revision #
ET_PASM		equ	80h		;entity type code

PASM		struc
ERLen		dword	sizeof(PASM)	;record length
EType		byte	ET_PASM		;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
Version		db	PAVer		;Assembly version #
flags		db	PAF_FRAME	;all views options flags
;
AName		db	32 dup (0)	;Print Assembly Name
;
;	Printer-Settings Information
;
dmFields		dword	0	;order,size same as DEVMODE
dmOrientation		word	0	;single partial block of data
dmPaperSize		word	0
dmPaperLength		word	0
dmPaperWidth		word	0
dmScale			word	0
dmCopies		word	0
dmDefaultSource		word	0
dmPrintQuality		word	0
dmColor			word	0
dmDuplex		word	0
dmYResolution		word	0
dmTTOption		word	0
;
PrName			db	32 dup (0) ;printer device name
;
HSize			dword	0	;horizontal size in 1 mm units
VSize			dword	0	;vertical size in 1 mm units
;
PrSwapColor	db	1		;color swap mode for printing
NrViews		word	0		;# view specifications that follow
;
;		0 or more view specification structures follow here
;
PASM		ends

PAVIEW		struc
RLen		dword	sizeof(PAVIEW)	;structure size
LocX1		Real4	0.0		;lower left in print area (mm)
LocY1		Real4	0.0
LocX2		Real4	0.0		;upper-right in print area (mm)
LocY2		Real4	0.0
VMode		dword	0		;view mode: 0=all, 1=named, 2=activeW
VScale		dword	0		;scaling: 0=fit to area, 1=scale
PDist		real4	1.0		;paperdist
DDist		real4	1.0		;drawing distance
VFlags		word	0		;view-specific flags
VName		db	32 dup (0)	;current view name
DName		db	256 dup (0)	;drawing file name
;
;	Reserved for future use:
;
VWidth		REAL4	10.0		;current view width
VHgt		REAL4	7.5		;current view height
VCenX		REAL4	5.0		;current view center X
VCenY		REAL4	3.75		;current view center Y
VCenZ		REAL4	0.0		;current view center Z
VRotXY		REAL4	0.0		;current view rotation about XY
VRotYZ		REAL4	0.0		;current view rotation about YZ
VRotZX		REAL4	0.0		;current view rotation about ZX
VType		dword	0		;view type bits
;
PAVIEW		ends

PInfoSize	equ	PASM.PrSwapColor-PASM.dmFields

;	PASM.flags values:

PAF_FRAME	equ	1		;1 = frame each view

;	PAVIEW.VFlags values:

PAV_DELETE	equ	8000h		;1=delete this view
PAV_FRAME	equ	1h		;1=draw frame around view
PAV_ALLBLK	equ	2h		;1=draw view in all black

;	===============================================================
;	NOTE: Note entity structure
;	===============================================================
NoteVer		equ	0		;structure version #

ET_Note		equ	81h		;entity type code

NOTE		struc
ERLen		dword	sizeof(NOTE)	;record length
EType		byte	ET_Note		;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
Version		db	NoteVer		;version #
flags		db	0		;options flags
;
NName		db	32 dup (0)	;Note Name
;
NData		db	0		;note data (ANSIZ multiline)
NOTE		ends

;	===============================================================
;	Sublist Markers
;	===============================================================
;	An entity that has a sublist of entities has the sublist
;	stored on disk immediately following its owner entity.
;	It starts with a Marker 0 record, followed by the entities
;	in the sublist, and ends with a Marker 1 entity.
;
;	Note that this is only when the data is stored in a file
;	image. When the drawing is in memory, sublists are accessed
;	through sublist functions.
;	---------------------------------------------------------------

Mark0		struc			;sublist start marker
ERLen		dword	5
MType		db	0
Mark0		ends

Mark1		struc			;sublist end marker
ERLen		dword	5
MType		db	1
Mark1		ends

;	===============================================================
;	Etype 41: SHEET Container Entity
;	===============================================================

ET_SHEET	equ	41		;entity type code
C_SHEET		equ	0
H_SHEET		equ	0

SHT_CUR		equ	1		;1 = sheet is current
SHT_VIS		equ	2		;2 = sheet is visible
SHT_NEW		equ	40000h		;4 = new add (for cancel)

SHEET		struc
ERLen		dword	sizeof(SHEET)	;record length
EType		byte	ET_SHEET	;entity type code
EFlags		byte	0		;erase/select bits
EFlags2		byte	0		;extra flags
EColor		byte	1		;entity color
EColor2		byte	1		;fill (2nd) color
EThick		byte	0		;pen thickness 0..25.4 mm
WPlane		word	0		;workplane (0 = XY plane)
ELayer		word	1		;layer
ELStyle		word	0		;line style (0=solid)
GroupID		word	0		;group id (0 = not grouped)
EFStyle		word	0		;fill style (0=hollow)
LWidth		real4	0.0		;line width
Tag		dword	0		;entity tag id
;
Status		dword	SHT_VIS		;Sheet status flags
;					;hi word = save for cancel of low word
;					;valid flag bits are low word only
SName		db	64 dup (0)	;sheet name (64 bytes always allocated)
SHEET		ends

