;	===============================================================
;	FONTS.CPY - Font Data Structures
;	===============================================================

;	Copyright 1994-1998 Evolution Computing
;	All rights reserved

;	===============================================================
;	Old Font InfoBlock Structure
;	===============================================================

;		Font table entry

Font		struct
ID		word	0
CharSet         SBYTE	?               ;ANSI_CHARSET etc.
PitchAndFamily  SBYTE	?               ;e.g. FF_SWISS or VARIABLE_PITCH
Hght            DWORD	?               ;sample height
Descent         DWORD	?               ;sample descent
InternalLeading DWORD	?               ;sample internal leading
Wdth            DWORD	?               ;sample width
ItalicWdth      DWORD	?               ;italic sample width
BoldWdth        DWORD	?               ;bold sample width
BoldItalicWdth  DWORD	?               ;bold italic sample width
MWdth           DWORD	?               ;max sample width
ItalicMWdth     DWORD	?               ;max italic sample width
BoldMWdth       DWORD	?               ;max bold sample width
BoldItalicMWdth DWORD	?               ;max bold italic sample width
FaceName	SBYTE	LF_FACESIZE dup (?)	;Typeface name
Font		ends

;		Old Font IB header

FontIB		struct
ERLen		dword	sizeof(FontIB)	;actual length (bytes in use)
EType		byte	ET_IB		;entity type code (always 0)
IType		byte	IB_FONT		;info block type (FF=XP)
Creator		word	0		;XPID (0=core)
AllocLen	dword	sizeof(FontIB)	;allocation length
Count		WORD	?
FontIB		ends

;	===============================================================
;	New Font InfoBlock Structure
;	===============================================================

;		New Font IB header

CurFontIBVer	equ	3		;IB Version #

FontIB2		struct
ERLen		dword	sizeof(FontIB)	;actual length (bytes in use)
EType		byte	ET_IB		;entity type code (always 0)
IType		byte	IB_FONT		;info block type (FF=XP)
Creator		word	0		;XPID (0=core)
AllocLen	dword	0		;allocation length
Count		WORD	0FFFFh		;marks new FontIB2
IBVer		word	CurFontIBVer	;IB Version #
NextID		word	256		;next font id to assign
FontIB2		ends

;	The font IB header above is followed by one or more
;	FONTX structures, then a dword = 0 to mark the end
;	of the infoblock.

;	---------------------------------------------------------------
;	Type 0 font record (FastCAD/DOS Vector .FNT)
;	---------------------------------------------------------------

FONT0		struc
RLen		dword	sizeof(FONT0)
ID		word	0		;font id #
TypeID		word	0		;type 0 font
;					;bit 8000h set = font unavailable
FntName		db	LF_FACESIZE dup (0) ;font name ANSIZ
;
Flags		db	0		;flag bits
IconID		db	3		;icon for display
		db	0,0
		dword	0,0,0		;reserved
hFont		dword	0		;font mem block handle
FONT0		ends

;	---------------------------------------------------------------
;	Type 1 font record (Windows TrueType)
;	---------------------------------------------------------------

FONT1		struc
RLen		dword	sizeof(FONT1)
ID		word	0		;font id #
TypeID		word	1		;type 1 font
;					;bit 8000h set = font unavailable
FntName		db	LF_FACESIZE dup (0) ;font name ANSIZ
;
Flags		db	0		;flag bits
IconID		db	0		;icon for display
		db	0,0		;reserved
		dword	0
lfHeight	dword	24h		;logfont std height
lfWidth		dword	0Eh		;logfont std width
CharSet		SBYTE	0		;ANSI_CHARSET, etc.
PitchAndFamily  SBYTE	17h		;e.g. FF_SWISS or VARIABLE_PITCH
HAdjust		real4	1.41176		;height multiplier=(sampleheight)
;					;/(sampleheight-descender)
FONT1		ends

;	---------------------------------------------------------------
;	Type 2 font record (ACAD Compatible Vector .SHX)
;	---------------------------------------------------------------

FONT2		struc
RLen		dword	sizeof(FONT2)
ID		word	0		;font id #
TypeID		word	2		;type 2 font
;					;bit 8000h set = font unavailable
FntName		db	LF_FACESIZE dup (0) ;font name ANSIZ
;
Flags		db	0		;flag bits
IconID		db	4		;icon for display
		db	0,0
		dword	0,0,0		;reserved
hFont		dword	0		;font mem block handle
FONT2		ends

;	===============================================================
;	Font Service Functions
;	===============================================================

FSVC_DRAW	equ	0		;draw text geometry
FSVC_PICK	equ	1		;pick test text geometry
FSVC_LEN	equ	2		;calc text geometry length
FSVC_BOX	equ	3		;EAX->tight-fit box around text
FSVC_EXBOX	equ	4		;EAX->margin box around text
FSVC_ADD	equ	5		;ESI->font name to add to IB
FSVC_INIT	equ	6		;ESI->FontIB2, just initialized
FSVC_END	equ	7		;ESI->FontIB2, release mem

;	===============================================================
;	FontID control flags
;	===============================================================

FID_UNAVAIL	equ	8000h		;font not available
FID_DUP		equ	4000h		;font duplicated
FID_MASK	equ	00FFh		;mask for valid type id

;	===============================================================
;	FONTX.Flags values
;	===============================================================

FF_Init		equ	1		;has been initialized
