;	===============================================================
;	FSTYLES.CPY - Fill Style Structures
;	===============================================================

;	Copyright 1996 Evolution Computing
;	All rights reserved
;	Written by Mike Riddle 12-1-96

;	===============================================================
;	FSTYIB: Fill Styles Information block header
;	===============================================================

;	CurHdr.FStyle = the currently selected style ID
;	It determines the currently selected fill style

CurFSTYIBVer	equ	1		;current version #

FSTYIB		struc
ERLen		dword	SIZEOF (FSTYIB)	;std info block header
EType		byte	ET_IB		;EType = Info Block
IType		byte	IB_FSTYLE	;IB type = Fill Styles
Creator		word	0		;XPID (0=core)
AllocLen	dword	0		;allocation length (0=ERLen)
;
IBVer		word	CurFSTYIBVer	;FSTYIB version id
NextID		word	256		;next ID to use
;
;				;fill style instance data starts here
;				;A record length of 0 ends the list
FSTYIB		ends

;	===============================================================
;	Individual FSTY style definitions
;	===============================================================

FS_PAT		equ	0	;brush pattern style
FS_HATCH	equ	1	;metric hatch style
FS_BMP		equ	2	;bitmap file style
FS_SYM		equ	3	;symbol fill style

FSTY		struc
rlen		dword	0	;style record length in bytes
fstype		sbyte	0	;style type id (FS_XXXX)
dofst		sbyte	0	;# bytes from rlen to data start
id		word	0	;fill style id (magic cookie)
flags		byte	0	;options flags
fsname		sbyte	0	;ANSIZ fill style name
FSTY		ends

;	Values for FSTY.flags:

FS_OUTL		equ	1	;outline the filled shape
FS_TRANS	equ	80h	;transparent color specified

;	---------------------------------------------------------------
;	Pattern styles have an 8 byte bitmap following the FSTY header
;	---------------------------------------------------------------

PatSty		macro	id,nm
		local	epat,spat
spat		dword	(epat-spat)+8
		sbyte	FS_PAT
		sbyte	(epat-spat)
		word	id
		sbyte	FS_OUTL
		sbyte	nm
		sbyte	0
epat		label	byte
		endm

;	---------------------------------------------------------------
;	Hatch Styles have 4*SIZEOF(HATCH) hatch data following the header
;	---------------------------------------------------------------

HATCH		struc
lsty		word	0	;0 = solid line
siz		real4	0.0	;0.0 = this set not drawn
ang		real4	0.0	;line bearing angle (degrees)
xofst		real4	0.0	;offset to start of line draw
yofst		real4	0.0	;(only affects styled lines)
HATCH		ends

NrHatch		equ	4

HatchSty	macro	id,nm
		local	epat,spat
spat		dword	(epat-spat)+(NrHatch*SIZEOF(HATCH))
		sbyte	FS_HATCH
		sbyte	(epat-spat)
		word	id
		sbyte	FS_OUTL
		sbyte	nm
		sbyte	0
epat		label	byte
		endm

;	---------------------------------------------------------------
;	Bitmap styles: data
;	---------------------------------------------------------------

BMPSTY		struc
fit		db	FS_EXPFILL	;fit code
tcolor		db	0		;transparent color
bmpname		db	0		;bmp file name (ANSIZ)
BMPSTY		ends

BmpSty		macro	id,nm,fit,fn
		local	epat,spat,fpat
spat		dword	(fpat-spat)
		sbyte	FS_BMP
		sbyte	(epat-spat)
		word	id
		sbyte	FS_OUTL
		sbyte	nm
		sbyte	0
epat		label	byte
		db	fit
		db	0
		sbyte	fn
		sbyte	0
fpat		label	byte
		endm

;	Bitmap fit code values

FS_TILE		equ	1
FS_EXPFILL	equ	2
FS_EXPFIT	equ	3
FS_COPY		equ	4

;	---------------------------------------------------------------
;	Symbol Styles
;	---------------------------------------------------------------

SYMSTY		struc
xstep		real4	1.0		;repeat stepping distance
ystep		real4	1.0
xscale		real4	1.0		;individual copy tforms
yscale		real4	1.0
xyrot		real4	0.0
symname		db	0		;bmp file name (ANSIZ)
SYMSTY		ends

SymSty		macro	id,nm,fit,fn
		local	epat,spat,fpat
spat		dword	(fpat-spat)
		sbyte	FS_SYM
		sbyte	(epat-spat)
		word	id
		sbyte	FS_OUTL
		sbyte	nm
		sbyte	0
epat		label	byte
		real4	1.0
		real4	1.0
		real4	1.0
		real4	1.0
		real4	0.0
		sbyte	fn
		sbyte	0
fpat		label	byte
		endm

