;	===============================================================
;	GEOMETRY.CPY - Real4 Geometry Structures
;	===============================================================
;	Copyright 1995 Evolution Computing
;	All rights reserved
;	Written by Mike Riddle 5-19-95
;	Additional work by Susan Montooth 5/95-11/95

;	Automatically included with global.cpy

;	===============================================================
;	Flags used by all geometry draw handlers (DCIR2 etc)
;	and by fill engine:

F_O	equ	01h			;outline this geometry
F_F	equ	02h			;fill this geometry
F_SO	equ	04h			;outline in solid pen
F_SF	equ	08h			;fill in solid brush

;	Flags used by SVC ENearT2 in EAX:

ENEARF_ANY	equ	0	;accept any point, even off end of ent
ENEARF_ON	equ	1	;point must be on the entity

;	---------------------------------------------------------------
;	2D point
;	---------------------------------------------------------------
;
GPnt2	STRUC
X	REAL4	0.0
Y	REAL4	0.0
GPnt2	ends

GDPnt2	STRUC
X	REAL8	0.0
Y	REAL8	0.0
GDPnt2	ends

DevP2	STRUC			;integer struc for DevPoint
x	dword	0
y	dword	0
DevP2	ENDS
;
;	---------------------------------------------------------------
;	3D point
;	---------------------------------------------------------------
;
GPnt3	STRUC
X	REAL4	0.0
Y	REAL4	0.0
Z	REAL4	0.0
GPnt3	ends
;
;	---------------------------------------------------------------
;	3D double precision point
;	---------------------------------------------------------------
;
GDPnt3	STRUC
X	REAL8	0.0
Y	REAL8	0.0
Z	REAL8	0.0
GDPnt3	ends
;
;	---------------------------------------------------------------
;	2D line
;	---------------------------------------------------------------
;
GLine2	STRUC
X1	REAL4	0.0	;first point
Y1	REAL4	0.0
X2	REAL4	1.0	;second point
Y2	REAL4	0.0
GLine2	ends

GDLine2	STRUC
X1	REAL8	0.0	;first point
Y1	REAL8	0.0
X2	REAL8	1.0	;second point
Y2	REAL8	0.0
GDLine2	ends

DevL2	STRUC				;integer line for DevLine
x1	dword	0
y1	dword	0
x2	dword	0
y2	dword	0
DevL2	ENDS

Ext2	STRUC
XLow	REAL4	0.0	;lower-left
YLow	REAL4	0.0
XHigh	REAL4	1.0	;upper-right
YHigh	REAL4	0.0
Ext2	ends

;	---------------------------------------------------------------
;	2D rectangle corners
;	---------------------------------------------------------------
;
GRect2	STRUC
X1	REAL4	0.0	;first point
Y1	REAL4	0.0
X2	REAL4	0.0	;second point
Y2	REAL4	0.0
GRect2	ends
;
;	---------------------------------------------------------------
;	3D rectangle corners
;	---------------------------------------------------------------
;
GRect3	STRUC
X1	REAL4	0.0	;first point
Y1	REAL4	0.0
Z1	REAL4	0.0
X2	REAL4	0.0	;second point
Y2	REAL4	0.0
Z2	REAL4	0.0
GRect3	ends
;
;	---------------------------------------------------------------
;	3D line
;	---------------------------------------------------------------
;
GLine3	STRUC
X1	REAL4	0.0	;first point
Y1	REAL4	0.0
Z1	REAL4	0.0
X2	REAL4	0.0	;second point
Y2	REAL4	0.0
Z2	REAL4	0.0
GLine3	ends
;
;	---------------------------------------------------------------
;	2D Arc
;	---------------------------------------------------------------
;
GArc2	STRUC
CenX	REAL4	0.0	;center
CenY	REAL4	0.0
Radius	REAL4	0.0	;radius
SAng	REAL4	0.0	;start angle, radians
AngW	REAL4	0.0	;arc width
GArc2	ends
;
;	---------------------------------------------------------------
;	2D circle
;	---------------------------------------------------------------
;
GCir2	STRUC
CenX	REAL4	0.0	;center
CenY	REAL4	0.0
Radius	REAL4	0.0	;radius
GCir2	ends
;
;	---------------------------------------------------------------
;	2d ellipse/elliptical arc
;	---------------------------------------------------------------
;
GElp2	STRUC
CenX	REAL4	0.0	;center
CenY	REAL4	0.0
Radius	REAL4	0.0	;major radius
Ecc	REAL4	0.0	;eccentricity minor/major (0..1)
Incl	REAL4	0.0	;major axis inclination angle 0..pi
SAng	REAL4	0.0	;start angle 0..2pi
AngW	REAL4	0.0	;arc width 0..2pi
GElp2	ends
;
;	---------------------------------------------------------------
;	2d/3d Node Lists (frame only)
;	---------------------------------------------------------------
;
GNL2	STRUC
Count	dw	0	;# nodes in list
Flags	db	0	;control flags
	db	0	;unused
X	REAL4	0.0	;first node
Y	REAL4	0.0
W	REAL4	0.0
GNL2	ends
;
GNL3	STRUC
Count	dw	0	;# nodes in list
Flags	db	NL_3D	;control flags
	db	0	;unused
X	REAL4	0.0	;first node
Y	REAL4	0.0
Z	REAL4	0.0
W	REAL4	0.0
GNL3	ends
;
;	Values for NFlags
;
NL_CLS	equ	80h	;1 = close node list (polygon)
NL_SLD	equ	40h	;1 = interior is solid (filled)
;	equ	20h	;reserved - must be 0
;	equ	10h	;reserved - must be 0
;	equ	08h	;reserved - must be 0
;
;	Bits 01h,02h,04h specify the type of node list:
;
NL_2D	equ	00h	;2D node list (8 bytes per node)
NL_2DW	equ	01h	;2D node list with W parameter (12 bytes per node)
NL_3D	equ	02h	;3D node list (12 bytes per node)
NL_3DW	equ	03h	;3D node list with W parameter (16 bytes per node)
;	equ	04h	;reserved
;	equ	05h	;reserved
;	equ	06h	;reserved
;	equ	07h	;reserved
;
GBox2	struc			;special case: 2D box
Count	dw	4		;# nodes in list
Flags	db	NL_CLS		;control flags
	db	0		;unused
X0	REAL4	0.0
Y0	REAL4	0.0
X1	REAL4	0.0
Y1	REAL4	0.0
X2	REAL4	0.0
Y2	REAL4	0.0
X3	REAL4	0.0
Y3	REAL4	0.0
GBox2	ends
;
;	---------------------------------------------------------------
;	2d/3d Path/Poly/Spline geometry (includes smoother type, 
;	start/end parameters)
;	---------------------------------------------------------------
;
GPath2	STRUC
SmType	db	SM_CB	;smoother type
SRes	db	0	;smoothing resolution
SParm	REAL4	0.0	;start parameter
EParm	REAL4	0.0	;end parameter
;---------------------------------------------- rest matches GNL2
Count	dw	0	;# nodes in list
Flags	db	0	;control flags
	db	0	;unused
X	REAL4	0.0	;first node
Y	REAL4	0.0
W	REAL4	0.0
GPath2	ends
;
GPath3	STRUC
SmType	db	SM_CB	;smoother type
SRes	db	0	;smoothing resolution
SParm	REAL4	0.0	;start parameter
EParm	REAL4	0.0	;end parameter
;---------------------------------------------- rest matches GNL3
Count	dw	0	;# nodes in list
Flags	db	NL_3D	;control flags
	db	0	;unused
X	REAL4	0.0	;first node
Y	REAL4	0.0
Z	REAL4	0.0
W	REAL4	0.0
GPath3	ends
;
;	Values for Smoother Flags
;
SM_NO	equ	0	;no smoothing
SM_CB	equ	1	;Cubic B-Spline (as in FastCAD 2.x)
SM_PB	equ	2	;Parabolic Blend (through-point)
SM_BZ	equ	3	;Bezier
;
;	---------------------------------------------------------------
;	2d Bezier (Special case of GPath2)
;	---------------------------------------------------------------
;
GBez2	STRUC
StParm	REAL4	0.0		;natural parameter start
EnParm	REAL4	1.0		;natural parameter end
Count	dw	4		;# nodes in list
Flags	db	NL_2D		;control flags
	db	0		;unused
X0	REAL4	0.0
Y0	REAL4	0.0
X1	REAL4	0.0
Y1	REAL4	0.0
X2	REAL4	0.0
Y2	REAL4	0.0
X3	REAL4	0.0
Y3	REAL4	0.0
GBez2	ends
;
;	---------------------------------------------------------------
;	2d text
;	---------------------------------------------------------------
;
GTxt2	STRUC
Font	dw	0		;text font id #
X	REAL4	0.0		;text origin
Y	REAL4	0.0
Hght	REAL4	0.0		;text height
XScl	REAL4	1.0		;text cell X scale (horizontal stretch)
BAng	REAL4	0.0		;text baseline angle
Spacing REAL4	0.0		;Inter-line spacing
TFlags	WORD	0		;style flags
DFlags	WORD	0		;data flags
Just	WORD	0		;text justification code
TData	SBYTE	0		;text data ASCIIZ string
GTxt2	ends

;	Text style flags (TFlags byte):

TS_VERT 	equ	1	;Vertical text flag
TS_FILL 	equ	2	;Filled text flag (unused in ECW)
TS_MIRROR	equ	4	;Mirrored text flag (unused in ECW)
				;		(unused in ECW)
TS_BOLD 	equ	16	;Bold text
TS_ITALIC 	equ	32	;Italic text
TS_UNDER	equ	64	;Underscore text
TS_STRIKE	equ	128	;Strikeout text
TS_OUTLINE	equ	256	;outline-only text
;
TS_METRICS	equ	8000h	;use new text metrics
TS_ANGFIXED	equ	512	;text angle doesn't transform

;	Text data flags (DFlags byte):

TD_FILE		equ	1	;data is file name string
TD_MACRO	equ	2	;data is macro variable name

;	Text justification codes (Just byte) as illustrated below

TJ_BL		equ	0	;bottom left
TJ_BC		equ	1	;bottom center
TJ_BR		equ	2	;bottom right
TJ_ML		equ	3	;mid left
TJ_MC		equ	4	;mid center
TJ_MR		equ	5	;mid right
TJ_TL		equ	6	;top left
TJ_TC		equ	7	;top center
TJ_TR		equ	8	;top right
TJ_XML		equ	9	;extended mid left
TJ_XMR		equ	10	;extended mid right
TJ_XXBL		equ	11	;extended bottom extended left
TJ_XBL		equ	12	;extended bottom left
TJ_XBC		equ	13	;extended bottom center
TJ_XBR		equ	14	;extended bottom right
TJ_XXBR		equ	15	;extended bottom extended right
TJ_XXTL		equ	16	;extended top extended left
TJ_XTL		equ	17	;extended top left
TJ_XTC		equ	18	;extended top center
TJ_XTR		equ	19	;extended top right
TJ_XXTR		equ	20	;extended top extended right

; 	     16---17----------18----------19---20
;	      |    |	       |	   |	|
; 	      |    6-----------7-----------8	|
;	      |    |	       |	   |	|
;	      |    |	       |	   |	|
;	      9----3-----------4-----------5---10
;	      |    |	       |	   |	|
;	      |    |	       |	   |	|
;	      |    0-----------1-----------2	|
;	      |    |	       |	   |	|
;	     11---12----------13----------14---15

