;	===============================================================
;	RECLIST.CPY - Record List Control Structure
;	===============================================================
;	Copyright 1994 Evolution Computing
;	All rights reserved
;	Written by Mike Riddle 8-14-94
;	Revised for WIN32 5-10-95 MR
;	---------------------------------------------------------------

; A Record is:

;    ===========================================================
;    | record length |                data bytes               |
;    ===========================================================
;    |              record length includes all bytes           |

;	A variable # of bytes that starts with a DWORD record length.
;	The length DWORD is included in the length. The smallest
;	valid record is 4 bytes long.

;    ---------------------------------------------------------------

; A Record LIST is:

;    ================================================================
;    | rec 1 |    rec 2   |  rec 3  | tos rec 4 |   unused memory   |
;    ================================================================
;    | Start                        | TOS                       Max |
;
;	The data structure specified by RECLIST contains pointers
;	to a block of global memory that contains none or more records
;	stored one after the other, with the newest record stored at
;	the highest offset, followed by available memory for a total
;	memory block size of RECLIST.Max bytes.

;	A record list is easily scanned from oldest to newest record.
;
;	---------------------------------------------------------------

; A Record STACK is:

;    ================================================================
;    |  unused memory   | tos rec 4 |  rec 3  |   rec 2   |  rec 1  |
;    ================================================================
;    | Start            | TOS                                   Max |
;
;	The data structure specified by RECSTACK contains pointers
;	to a block of global memory that contains none or more records
;	stored one below the other, with the newest record stored at
;	the lowest offset, preceeded by available memory for a total
;	memory block size of RECLIST.Max bytes.

;	A record stack is easily scanned from newest to oldest record.
;
;	===============================================================

RECLIST		STRUC
Max		dword	0		;max offset of list +1
Start		dword	0		;offset of start of data
TOS		dword	0		;current TOS (top of stack)
;					; offset of top valid record
EOL		dword	0		;offset of 1st free byte after
;					; TOS record
flags		dword	0		;0=RECLIST
RECLIST		ENDS

RECSTACK	STRUC
Max		dword	0		;max offset of stack +1
Start		dword	0		;offset of start of data
;					;0=not yet allocated
TOS		dword	0		;current TOS (top of stack)
;					; offset of top valid record
EOL		dword	0		;offset of 1st free byte after
;					; TOS record
flags		dword	1		;1=RECSTACK
RECSTACK	ENDS

