;	===============================================================
;	8087.CPY - common 8087 macros
;	===============================================================
;	Copyright 1986 Evolution Computing
;	All rights reserved
;	Written by Mike Riddle 1/7/1986
;	---------------------------------------------------------------

	externdef	s87:word
;
fstat	macro
	if		cpu87 EQ c87
	fstsw		s87
	fwait
	mov		ah,byte ptr s87+1
	sahf
	endif
	if		cpu87 GE c287
	fstsw		ax
	fwait
	sahf
	endif
	endm
;
fjl	macro	jto
	fstat
	jb	jto
	endm
;
fjle	macro	jto
	fstat
	jbe	jto
	endm
;
fje	macro	jto
	fstat
	je	jto
	endm
;
fjne	macro	jto
	fstat
	jne	jto
	endm
;
fjg	macro	jto
	fstat
	ja	jto
	endm
;
fjge	macro	jto
	fstat
	jae	jto
	endm
;
fj3	macro	jtol,jtoe,jtog
	fstat
	jb	jtol
	ja	jtog
	jmp	jtoe
	endm

;	Use FINVAL after divide to protect against divides by numbers
;	that are nearly zero relative to numerator. You must first 
;	ensure denominator not exactly zero, and use fclex 
;	to clear exception flags.

finval	macro	jto		;;jump if divide result is invalid
	IF	cpu87 GE c287	;check exception flags
	fstsw	ax
	fwait
	ELSE
	fstsw	s87
	fwait
	mov	al,byte ptr s87
	ENDIF
	and	al,11100B	;mask out all but UE, OE, and ZE
	lahf			;set carry if divide OK
	shl	ah,1
	shl	ah,1
	jnc	jto
	endm

;
fdup	macro
	fld	st(0)
	endm
;
fpop	macro
	fstp	st(0)
	endm

;	macros for floating point numbers in memory

fmove	macro	dest,src
	if	cpu87 NE e87
	fwait
	endif
	MOVED	dest,src
	endm

fdmove	macro	dest,src
	if	cpu87 NE e87
	fwait
	endif
	fndmove	dest,src
	endm

fnmove	macro	dest,src
	MOVED	dest,src
	endm

fndmove	macro	dest,src
	mov	eax,dword ptr src
	mov	dword ptr dest,eax
	mov	eax,dword ptr src+4
	mov	dword ptr dest+4,eax
	endm

fswap	macro	A,B			;exchanges 2 reals in memory
	fwait
	mov	ax,word ptr A
	xchg	word ptr B,ax
	mov	word ptr A,ax
	mov	ax,word ptr A+2
	xchg	word ptr B+2,ax
	mov	word ptr A+2,ax
	endm
;
fnswap	macro	A,B			;exchanges 2 reals in memory
	mov	ax,word ptr A
	xchg	word ptr B,ax
	mov	word ptr A,ax
	mov	ax,word ptr A+2
	xchg	word ptr B+2,ax
	mov	word ptr A+2,ax
	endm
;
fzero	macro	A
	mov	dword ptr A,0
	endm
;
fdzero	macro	A
	mov	dword ptr A,0
	mov	dword ptr A+4,0
	endm
;
fone	macro	A
	mov	dword ptr A,3F800000h
	endm
;
fjzero	macro	var,lbl			;jump to label if real var=0
	test	byte ptr var+3,7Fh
	jz	lbl
	endm

fdjzero	macro	var,lbl			;jump to label if real var=0
	test	byte ptr var+7,7Fh
	jz	lbl
	endm

fjnzero	macro	var,lbl			;jump to label if real var!=0
	test	byte ptr var+3,7Fh
	jnz	lbl
	endm

fjone	macro	var,lbl			;jump to label if real var=0
	cmp	var,03F800000h
	jz	lbl
	endm

fjnone	macro	var,lbl			;jump to label if real var!=0
	cmp	var,03F800000h
	jnz	lbl
	endm

fjpos	macro	var,lbl			;jump to label if real var  0
	cmp	byte ptr var+3,80h
	jbe	lbl
	endm

fjneg	macro	var,lbl			;jump to label if real var < 0
	cmp	byte ptr var+3,80h
	ja	lbl
	endm

fjsdif	macro	var1,var2,lbl		;jump to label if signs differ
	mov	al,byte ptr var1+3	;isolate sign bits in al and ah
	and	al,80h
	mov	ah,byte ptr var2+3
	and	ah,80h
	xor	al,ah
	jnz	lbl
	endm

fpos	macro	var			;force real variable to positive
	and	byte ptr var+3,NOT 80h
	endm

fneg	macro	var			;force real variable negative
	or	byte ptr var+3,80h
	endm

fmax	macro	A
	local	ismax
	IFB	<A>
	fcom	st(1)			;st=max(st,st(1))
	fjl	ismax
	fxch	st(1)
ismax:	fpop
	ELSE
	fcom	A			;ST=MAX(ST,A)
	fjge	ismax
	fpop
	fld	A
ismax:
	ENDIF
	endm
;
fmin	macro	A
	local	ismin
	IFB	<A>
	fcom	st(1)			;st=min(st,st(1))
	fjg	ismin
	fxch	st(1)
ismin:	fpop
	ELSE
	fcom	A			;ST=MIN(ST,A)
	fjle	ismin
	fpop
	fld	A
ismin:
	ENDIF
	endm

;	load /2

fldpi2	MACRO
	fld1
	fchs
	fldpi
	fscale
	fstp	st(1)
	ENDM
;
;	load /4

fldpi4	MACRO
	fld1
	fadd	st,st(0)
	fchs
	fldpi
	fscale
	fstp	st(1)
	ENDM

;	round st(0) up (instead of toward nearest EVEN integer)

frndup	MACRO
	fstcw	s87
	fwait
	push	s87		;save current state
	and	s87,0f3ffh	;mask rounding control bits
	or	s87,00800h	;round toward +infinity
	fldcw	s87
	frndint			;do the rounding
	pop	s87
	fldcw	s87		;restore original control state
	ENDM

;	round st(0) down (instead of toward nearest EVEN integer)

frnddn	MACRO
	fstcw	s87
	fwait
	push	s87		;save current state
	and	s87,0f3ffh	;mask rounding control bits
	or	s87,00400h	;round toward -infinity
	fldcw	s87
	frndint			;do the rounding
	pop	s87
	fldcw	s87		;restore original control state
	ENDM

;	init floating-point processor or emulator

EINIT	macro
	if	(cpu87 EQ win87)
	extern	Em87Reset:stdcall
	call	Em87Reset
	else
	finit
	endif
	endm

;	extract exponent and significand

EXTRACT macro
	if	(cpu87 EQ win87)
	extern	Em87Xtract:far
	call	Em87Xtract
	else
	fxtract
	endif
	endm

;	Detect not-a-number in memory

fjnan	macro	A,Lbl
	cmp	byte ptr A+3,0FFh
	jz	Lbl
	endm
